// rcvUDPHBWin.cpp (V1.0): the receiver of UDPHB in MS windows.
//
// Author:  Huahui Wu
//          flashine@gmail.com
//
// History
//   Version 1.0 - 2005 Nov 1
//
// Description:
//   rcvUDPHBWin.cpp
//		Very simple, Works in conjunction with sndUDPHBWin.cpp as 
//		a Windows sender or sndUDPHBLnx as a Linux sender.
//		The program sets itself up as a receiver using the UDP
//		protocol. It waits for UDP data from a sender, displays
//		the incoming connection, bounce the ping packet back 
//		to the sender.
//
// Compile and Link: 
//	 Use MS Visual C++ .NET
//	 Compile and link with wsock32.lib
//
// Run: 
//   Pass the port number that the receiver should bind() to
//   on the command line. Any port number not already in use
//   can be specified.
//   If want to measure one way delay, add "oneway" as the second arg
//	 Example: rcvUDPHBWin 2000 - works like a normal Ping
//			  rcvUDPHBWin 2000 oneway - measures one way delay			
//
// License: 
//   This software is released into the public domain.
//   You are free to use it in any way you like.
//   This software is provided "as is" with no expressed
//   or implied warranty.  I accept no liability for any
//   damage or loss of business that this software may cause.
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "winsock.h"
#include <signal.h>

#define MAX_PKT_SIZE 10240
#define MAX_PKT_NUM 1000

struct Ping_Pkt{
	int seq;
	ULARGE_INTEGER counter;
	unsigned char padding[MAX_PKT_SIZE];
};

// Function prototype
void Datagramreceiver(short nPort, bool oneway);	// the receiver 
void CleanUp(int);						// handle for ctrl_c	
ULARGE_INTEGER GetCurTimeInMs(); //get current time in milli second


// Helper macro for displaying errors
#define PRINTERROR(s)	\
		fprintf(stderr,"\n%s: %d\n", s, WSAGetLastError())

int pktRcvd;
ULARGE_INTEGER initCounter; // the counter when PING starts
ULARGE_INTEGER prevCounter; // the counter of previous packet
ULARGE_INTEGER	curCounter;
ULARGE_INTEGER   frequency;
int minDelay, maxDelay; // the minum and the maximum delay

void main(int argc, char **argv)
{
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA wsaData;
	int nRet;
	short nPort;
	bool oneway = false;
	pktRcvd = 0;
	initCounter.QuadPart = 0;
	prevCounter.QuadPart = 0;

	//
	// Check for port argument
	//
	if (argc < 2 || argc > 3)
	{
		fprintf(stderr,"\nUsage: %s PortNumber\n", argv[0]);
		fprintf(stderr,"\n or %s PortNumber oneway\n", argv[0]);
		return;
	}
	if (argc == 3) oneway = true;
	
	nPort = atoi(argv[1]);
	
	//
	// Initialize WinSock and check version
	//
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested)
	{	
		fprintf(stderr,"\n Wrong version\n");
		return;
	}


	signal(SIGINT, CleanUp);
	//
	// Do all the stuff a datagram receiver does
	//
	Datagramreceiver(nPort, oneway);
	
	//
	// Release WinSock
	//
	WSACleanup();
}

////////////////////////////////////////////////////////////

void Datagramreceiver(short nPort, bool oneway)
{
	//
	// Create a UDP/IP datagram socket
	//
	SOCKET theSocket;

	theSocket = socket(AF_INET,		// Address family
					   SOCK_DGRAM,  // Socket type
					   IPPROTO_UDP);// Protocol
	if (theSocket == INVALID_SOCKET)
	{
		PRINTERROR("socket()");
		return;
	}

	
	//
	// Fill in the address structure
	//
	SOCKADDR_IN sareceiver;

	sareceiver.sin_family = AF_INET;
	sareceiver.sin_addr.s_addr = INADDR_ANY; // Let WinSock assign address
	sareceiver.sin_port = htons(nPort);	   // Use port passed from user


	//
	// bind the name to the socket
	//
	int nRet;

	nRet = bind(theSocket,				// Socket descriptor
				(LPSOCKADDR)&sareceiver,  // Address to bind to
				sizeof(struct sockaddr)	// Size of address
				);
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("bind()");
		closesocket(theSocket);
		return;
	}


	//
	// This isn't normally done or required, but in this 
	// example we're printing out where the receiver is waiting
	// so that you can connect the example sender.
	//
	int nLen;
	nLen = sizeof(SOCKADDR);
	char szBuf[MAX_PKT_SIZE];

	nRet = gethostname(szBuf, sizeof(szBuf));
	if (nRet == SOCKET_ERROR)
	{
		PRINTERROR("gethostname()");
		closesocket(theSocket);
		return;
	}

	//
	// Show the receiver name and port number
	//
	printf("\nreceiver named %s waiting on port %d\n",
			szBuf, nPort);
			

	//
	// Wait for data from the sender
	//
	SOCKADDR_IN sasender;
	int tSnd, tRcv;
	int delay;
	int tSent; // the time when each packet was sent in millisecond
	double tInterval; // the difference between the time when two packets were received

	while(1){
		memset(szBuf, 0, sizeof(szBuf));
		nRet = recvfrom(theSocket,				// Bound socket
						szBuf,					// Receive buffer
						sizeof(szBuf),			// Size of buffer in bytes
						0,						// Flags
						(LPSOCKADDR)&sasender,	// Buffer to receive sender address 
						&nLen);					// Length of sender address buffer

		pktRcvd ++;

		if (oneway){

			Ping_Pkt* ping_pkt=(Ping_Pkt*)szBuf; 

			//get cpu frequency and current counter
			curCounter = GetCurTimeInMs();

			//get one way delay in millisecond
			delay =curCounter.QuadPart - ping_pkt->counter.QuadPart;

			if (initCounter.QuadPart == 0){ //if this is the first packet
				initCounter = curCounter;
				tRcv = 0;
				tSnd = 0-delay;

				minDelay = delay;
				maxDelay = delay;
			}else {
				tRcv = curCounter.QuadPart - initCounter.QuadPart;
				tSnd = ping_pkt->counter.QuadPart - initCounter.QuadPart;

				if (delay < minDelay)
					minDelay = delay;
				if (delay > maxDelay)
					maxDelay = delay;
			}

			tInterval = curCounter.QuadPart - prevCounter.QuadPart;
			if (tInterval < 1)
				tInterval = 1;

			//
			// Display the data that was received
			//
			printf("bytes=%d seq=%d tSnd=%dms tRcv=%dms delay=%dms\n",
				nRet, ping_pkt->seq, tSnd, tRcv, delay, 1.0*nRet/tInterval);

			prevCounter = curCounter;
			
		}else{
		//
		// Show that we've received some data
		//
		printf("\n%d bytes received\n", nRet);
		//
		// Send data back to the sender
		//
		sendto(theSocket,					// socket
			szBuf,							// Send buffer
			nRet,							// Length of data to be sent
			0,								// Flags
			(LPSOCKADDR)&sasender,			// Address to send data to
			nLen);							// Length of address

		}
	}
	closesocket(theSocket);
	return;
}

void CleanUp(int arg1){
	double tTotal = curCounter.QuadPart - initCounter.QuadPart;
	printf("receiver is now off\n");
	printf("Network Delay:\n");
	printf("\tMin = %dms, Max = %dms, Difference = %dms\n", minDelay, maxDelay, maxDelay - minDelay);
	printf("Receiving Bandwidth:\n");
	printf("\t%d packets are received in %8.3f milli second\n", pktRcvd, tTotal);
	printf("\tAverage receiving bandwidth is %8.3f packets per second\n", 1000*pktRcvd/tTotal);
	exit(0);
}

ULARGE_INTEGER GetCurTimeInMs(){
	SYSTEMTIME  sysTime;
	FILETIME	fileTime;
	ULARGE_INTEGER timeInMs;
	::GetSystemTime(&sysTime);
	::SystemTimeToFileTime(&sysTime,&fileTime);
	timeInMs.LowPart = fileTime.dwLowDateTime;
	timeInMs.HighPart = fileTime.dwHighDateTime;
	timeInMs.QuadPart /= 10000; // since the filetime is 100-nanosecond based.
								// but we need milli second based
	return timeInMs;
}